#include <mysql/mysql.h>
#include <pwd.h>
#include "mysql_queries.h"
#include "qsutil.h"
#include "case.h"
#include "str.h"
#include "stralloc.h"

stralloc user = { 0 };
stralloc home = { 0 };
stralloc shell = { 0 };

extern MYSQL dbh, *mysql;
extern MYSQL_RES *result;

extern MYSQL_ROW do_query(stralloc *query);

struct passwd mypw;

/* get a pw entry from the database                                           */
/* returns: struct passwd if the user logged in, 1 if the user got password   */
/* wrong or 2 if the user didn't exist (-1 on error)                          */
struct passwd *getpw_mysql(char *username, char *pass) {
  int i, len, uid, gid;
  MYSQL_ROW row;
  stralloc query = { 0 };
  stralloc passwd = { 0 };
  char *bogus;

  i = str_len(username);

  if (! stralloc_ready(&user, 2 * i + 1)) return -1;
  len = str_len(POPSEL) + mysql_escape_string(user.s, username, i) + 2;
  if (! stralloc_ready(&query, len)) return -1;
  if (! stralloc_cats(&query, POPSEL)) return -1;
  if (! stralloc_cats(&query, user.s)) return -1;
  if (! stralloc_append(&query, "'")) return -1;
  if (! stralloc_0(&query)) return -1;
  
  row = do_query(&query);
  if ((int) row == -1) return -1;
  if ((int) row == 0) {
#ifndef O_NOT_LOG
    log2(username, " does not exist in database\n");
#endif
    /* result has already been freed by do_query() */
    disconnect_mysql();
    return 0;
  }

  if (str_diff(row[1], "N")) {
#ifndef O_NOT_LOG
    log2(username, " is suspended\n");
    mysql_free_result(result);
    disconnect_mysql();
    return 1;
  }
#endif

#define ZARKS {\
    mysql_free_result(result); \
    disconnect_mysql(); \
    return 1; \
}

  case_lowers(row[0]);
  if (! str_diff(row[0], "password")) {
    if (str_diff(row[2], pass)) {
#ifndef O_NOT_LOG
      log2(username, " got password wrong\n");
#endif
      ZARKS
    }
  }
  else if (! str_diff(row[0], "crypt-password")) {
    if (str_diff(row[2], crypt(pass, row[2]))) {
#ifndef O_NOT_LOG
      log2(username, " got crypt-password wrong\n");
#endif
      ZARKS
    }
  }
  else if (! str_diff(row[0], "mysql-password")) {
    if (! stralloc_ready(&passwd, 17)) return -1;
    make_scrambled_password(passwd.s, pass);
    if (str_diff(row[2], passwd.s)) {
#ifndef O_NOT_LOG
      log2(username, " got mysql-password wrong\n");
#endif
      ZARKS
    }
  }
  else {
    log2("MySQL misconfiguration: invalid password type for ", username);
    ZARKS
  }

  uid = strtol(row[3], &bogus, 0);
  if ((int) *bogus) {
    log3("MySQL misconfiguration: uid should be a number for ", username, "!\n");
    ZARKS
  }
  gid = strtol(row[4], &bogus, 0);
  if ((int) *bogus) {
    log3("MySQL misconfiguration: gid should be a number for ", username, "!\n");
    ZARKS
  }

  if (! stralloc_ready(&home, str_len(row[5]))) return -1;
  if (! stralloc_copys(&home, row[5])) return -1;
  if (! stralloc_0(&home)) return -1;
  if (! stralloc_ready(&shell, str_len(row[6]))) return -1;
  if (! stralloc_copys(&shell, row[6])) return -1;
  if (! stralloc_0(&shell)) return -1;

  mysql_free_result(result);
  disconnect_mysql();

  mypw.pw_name = user.s;
  mypw.pw_uid = uid;
  mypw.pw_gid = gid;
  mypw.pw_dir = home.s;
  mypw.pw_shell = shell.s;
  return &mypw;
}
